/*
 * Copyright 2006. Lluis Pamies i Juarez
 * Universitat Rovira i Virgili
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * Creates a callaback to an object method.
 * Source: http://www.deepwood.net/writing/method-references.html.utf8
 */
function createMethodReference(object, methodName) {
    return function () {
        object[methodName].apply(object, arguments);
    };
}

/*
 * A column is defined with its width and the x coordinate
 */
function Column(id, num) {
	this.id = id;
	this.num = num;
	this.width = 0;
	this.x_ini = 0;
	this.margin = 5;
	this.h_offset = 163;
	this.portlets = new Array();
}

// Add a portlet to the appropiate position
Column.prototype.addPortlet = function(port, row) {
	if(row!=null) {
		var i=0;
		while(i < this.portlets.length && this.portlets[i].row < row) {
			i++;
		}
		this.portlets.splice(i,0,port)
	}else{
		port.row = this.portlets.push(port)
		this.updateRows()
	}
	port.column = this;
}

Column.prototype.delPortlet = function(port) {
	for(var i=0; i<this.portlets.length; i++)
		if(this.portlets[i] == port)
			this.portlets.splice(i, 1)
	this.updateRows()
}

Column.prototype.updatePortlet = function(port) {
	this.updateRows()
}

Column.prototype.updateRows = function() {
	for(j=1; j<this.portlets.length; j++) {
		for(i=0; i<this.portlets.length-j; i++) {
			if(this.portlets[i].getY() > this.portlets[i+1].getY()) {
				tmp = this.portlets[i+1]
				this.portlets[i+1] = this.portlets[i]
				this.portlets[i] = tmp
			}
			this.portlets[i].row = i
			this.portlets[i+1].row = i+1
		}
	}
}

// Draw their portlets
Column.prototype.draw = function() {
	y = this.h_offset;
	for(i=0; i<this.portlets.length; i++) {
		top = this.portlets[i]
		top.resizeTo(this.width,top.getHeight());
		top.moveTo(this.x_ini, y);
		top.setPopup(false);
		top.Drop()
		y += top.getHeight()+2*this.margin;
	}
}

Column.prototype.showAll = function() {
	for(i=0; i<this.portlets.length; i++) {
		this.portlets[i].showAll();
	}
}

Column.prototype.ownsPortlet = function(port) {
	center = (port.getX()+(port.getWidth()/2));
	if(center<0 && this.id==0) return true
	else if(center>dd.getWndW() && this.id==this.num-1) return true
	return (center > this.x_ini-this.margin) && (center < this.x_ini+this.width+this.margin)
}

Column.prototype.sendPositions = function() {
	for(i=0; i<this.portlets.length; i++) {
		this.portlets[i].sendPosition();
		this.portlets[i].sendSize();
	}
}


/*
 * The workspace contains all the portlets and columns and 
 * display they correctly.
 */
function Workspace() {
	//All portlets
	this.portlets = new Array();

	//Floating portlets (ordered by top down...)
	this.floating = new Array();

	//General parameters
	this.dragging = false;
	this.len = 0;
	this.num_cols = 4;
	this.col_margin = 10;
	this.h_offset = 100;

	// Initialize columns
	this.columns = new Array(this.num_cols);
	this.col_width = (dd.getWndW()-(this.num_cols+1)*this.col_margin)/this.num_cols;
	for(i=0; i<this.num_cols; i++) {
		this.columns[i] = new Column(i, this.num_cols);
		this.columns[i].width = this.col_width;
		this.columns[i].x_ini = (this.col_width*i)+this.col_margin*(i+1)
	}

	// Initiaite wz_dragdrop library
	SET_DHTML()
}

Workspace.prototype.portletExists = function(port) {
	for(i=0; i<this.portlets.length; i++) {
		if(this.portlets[i].name==port.name) return true;
	}
	return false;
}

Workspace.prototype.addPortlet = function(title, url, row, col, height, status, type) {
	port = new Portlet(title, url, type, '_portlet_'+this.len, row, col, 10);
	if(this.portletExists(port)) return;
	
	this.len++;
	if(col<0 || col>=this.num_cols)	alert(title+" column out of range")
	
	this.columns[col].addPortlet(port, row)
	this.portlets.push(port)

	port.print(status);

	// Get the parameters to initialize the wz_dragdrop library.
	prms = port.getParams();
	for(i=0; i<prms.length; i++){
		ADD_DHTML(prms[i])
	}
	port.init();
	port.resizeTo(port.getWidth(), height);
	
	return port;
}

Workspace.prototype.restorePortlet = function() {
	sel = document.getElementById('restore_select');
	opt = sel.options[sel.selectedIndex];
	parts = opt.value.split('$%&');
	port = ws.addPortlet(opt.label, parts[0], 0, 0, 200, 'view', parts[1].toLowerCase());
	if(port!=null) {
		ws.init();
		this.columns[0].sendPositions()
		port.sendState('view')
	}
}

Workspace.prototype.init = function(portal) {
	this.portal = portal
	for(var i=0; i<this.num_cols; i++) {
		this.columns[i].showAll();
	}
	this.update();
}

Workspace.prototype.update = function() {
	for(var i=0; i<this.num_cols; i++) {
		this.columns[i].draw();
	}
}

Workspace.prototype.onResize = function() {
	// Resize columns
	this.col_width = (dd.getWndW()-(this.num_cols+1)*this.col_margin)/this.num_cols;
	for(var i=0; i<this.num_cols; i++) {
		this.columns[i].width = this.col_width;
		this.columns[i].x_ini = (this.col_width*i)+this.col_margin*(i+1)
	}
	
	// Resize and move portlets
	this.update();
}

Workspace.prototype.updateColumn = function(port) {
	for(var i=0; i<this.num_cols; i++) {
		if(this.columns[i].ownsPortlet(port)) return i
	}
}

Workspace.prototype.Drop = function() {
	p = dd.obj._portlet

	// The portlet is attached to a column
	if(p.column!=null){
		//Move portlets from columns and rows
		col = this.updateColumn(p)
		old = p.column.id
		if(col == p.column.id) {
			this.columns[col].updatePortlet(p)
		} else {
			this.columns[p.column.id].delPortlet(p)
			this.columns[col].addPortlet(p)
		}
	
		//Update all
		this.columns[col].sendPositions()
		this.columns[old].sendPositions()
		p.Drop()

	// The portlet is floating
	}else{
		p.Drop()
		
		//Move the portlet to top.
		//Delete portlet from floating array
		for(i=0; i<this.floating.length; i++) {
			if(this.floating[i]==port) {
				this.floating.splice(i,1)	
			}
		}
		//Adding portlet to floating array (at top)
		this.floating.push(port)

		//Change Z index of floating portlets
		for(i=this.floating.length; i>0; i--) {
			//TODO !!!
			this.floating[i-1].setZ(i*10)
		}
	}
	
	//Enable dragable again
	for(i=0; i<this.portlets.length; i++) {
		this.portlets[i].enableDraggable()
	}

	this.update();
	this.dragging = false
}

Workspace.prototype.Drag = function() {
	p = dd.obj._portlet
	p.Drag()
	
	//Disable draggable for all portlets.
	//I need to do it, because when the pointer passes over other portlet the
	//onmove function changes.
	if(this.dragging == false) {
		for(i=0; i<this.portlets.length; i++) {
			if(this.portlets[i]!=p) this.portlets[i].disableDraggable()
		}
		this.dragging = true
	}
}

Workspace.prototype.getPortlet = function(id) {
	for(i=0; i<this.portlets.length; i++) {
		if(this.portlets[i].id==id) return this.portlets[i];
	}
	return null;
}

Workspace.prototype.edit = function(id) {
	port = this.getPortlet(id);
	if(port) {
		port.edit();
	}
}

Workspace.prototype.view = function(id) {
	port = this.getPortlet(id);
	if(port) {
		port.view();
	}
}

Workspace.prototype.help = function(id) {
	port = this.getPortlet(id);
	if(port) {
		port.help();
	}
}

Workspace.prototype.close = function(id) {
	port = this.getPortlet(id);
	if(port.column!=null) {
		port.column.delPortlet(port);
	}
	port.close();

	var i=0;
	while(i<this.portlets.length && this.portlets[i].id!=id) i++
	if(i!=this.portlets.length) this.portlets.splice(i,1);

	this.update()
}

Workspace.prototype.popUp = function(id) {
	port = this.getPortlet(id);
	
	// The portlet is attached to a column
	if(port.column != null) {
		port.old_height = port.getHeight();
		port.popUp()
		port.column.delPortlet(port)
		port.old_column = port.column
		port.column = null
	
		//Adding portlet to floating array
		this.floating.push(port)
	
	// The portlet is floating
	}else{
		port.resizeTo(port.getWidth(), port.old_height)
		port.column = port.old_column
		port.column.addPortlet(port)

		//Delete portlet from floating array
		for(i=0; i<this.floating.length; i++) {
			if(this.floating[i]==port) {
				this.floating.splice(i,1)	
			}
		}
	}
	this.update();
}

/*
 * This class is to abstract the wz_dragdrop library that
 * allows only one window, and add a new features to it.
 */
function Portlet(title, src, type, id, row, col, z) {
	this.name = title;
	this.id = id;
	this.type = type;
	this.src = src;
	this.title_h = 15;
	if(type=='web')
		this.tools_w = 32;
	else
		this.tools_w = 80;
	this.dragging = false;
	this.popup = false;
	this.z = z;
	this.column = null
	this.row = row
}

Portlet.prototype.print = function(status, height) {
	// Prints the document
	a = '<div id="port_'+this.id+'_main" class="window"></div>'
	a+= '<div id="port_'+this.id+'_title" class="window_title">'+this.name+'</div>'
	a+= '<div id="port_'+this.id+'_tools" class="window_tools">'+this.getTools()+'</div>'
	if(this.type == 'p2p')
		url = this.src+'/'+status+'.jsp'
	else
		url = this.src
	a+= '<iframe id="port_'+this.id+'_hidden_frame" style="width: 0; height: 0; visibility: hidden" src="'+url+'"></iframe>'
	a+= '<iframe id="port_'+this.id+'_frame" class="window_content" src="loading.htm"></iframe>'
	a+= '<div id="port_'+this.id+'_corner" class="resize_button"><img src="images/corner.gif" /></div>'
	this.div = document.createElement('div')
	this.div.id = this.id
	this.div.innerHTML = a
	document.body.appendChild(this.div)
	
	this.loader = document.getElementById('port_'+this.id+'_hidden_frame')
	this.loader.onload = createMethodReference(this, 'loadData')
	this.content = document.getElementById('port_'+this.id+'_frame');
}

Portlet.prototype.loadData = function() {
	this.content.src = this.loader.src
}

Portlet.prototype.getTools = function() {
	tools = '';
	if(this.type=='p2p'){
		tools += '<span onclick="ws.view(\''+this.id+'\')"><img src="images/view.gif" /></span>';
		tools += '<span onclick="ws.edit(\''+this.id+'\')"><img src="images/edit.gif" /></span>';
		tools += '<span onclick="ws.help(\''+this.id+'\')"><img src="images/help.gif" /></span>';
	}
	tools += '<span onclick="ws.popUp(\''+this.id+'\')"><img src="images/max.gif" /></span>';
	tools += '<span onclick="ws.close(\''+this.id+'\')"><img src="images/close.gif" /></span>';
	return tools;
}

Portlet.prototype.view = function() {
	this.loader.src = this.src + '/view.jsp';
	this.loader.onload = createMethodReference(this, 'loadData')
	this.sendState('view')
}

Portlet.prototype.edit = function() {
	this.loader.src = this.src + '/edit.jsp';
	this.loader.onload = createMethodReference(this, 'loadData')
	this.sendState('edit')
}

Portlet.prototype.help = function() {
	this.loader.src = this.src + '/help.jsp';
	this.loader.onload = createMethodReference(this, 'loadData')
	this.sendState('help')
}

Portlet.prototype.close = function() {
	document.body.removeChild(this.div);
	this.sendState('close')
}

Portlet.prototype.getParams = function() {
	var args = new Array(5);
	args[0] = "port_"+this.id+"_main"+CURSOR_MOVE;
	args[1] = "port_"+this.id+"_title"+NO_DRAG;
	args[2] = "port_"+this.id+"_tools"+NO_DRAG;
	args[3] = "port_"+this.id+"_frame"+NO_DRAG;
	args[4] = "port_"+this.id+"_corner"+MAXOFFLEFT+0+MAXOFFTOP+0+CURSOR_NW_RESIZE;
	return args;
}

Portlet.prototype.getHeight = function() {
	return this.main.h;
}

Portlet.prototype.getWidth = function() {
	return this.main.w;
}

Portlet.prototype.getX = function() {
	return this.main.x;
}

Portlet.prototype.getY = function() {
	return this.main.y;
}

Portlet.prototype.moveTo = function(x,y) {
	this.main.moveTo(x,y);
}

Portlet.prototype.resizeTo = function(w,h) {
	this.main.resizeTo(w,h);
	this.frame.resizeTo( this.main.w, this.main.h-this.title_h );
	this.frame.moveTo( this.main.x, this.main.y+this.title_h+1 );
	this.title.resizeTo( this.main.w, this.title_h );
	this.title.moveTo( this.main.x, this.main.y+1 );
	this.tools.resizeTo( this.tools_w, this.title_h );
	this.tools.moveTo( this.main.x+this.main.w-this.tools_w, this.main.y+1 );
	this.corner.moveTo( this.main.x+this.main.w-this.corner.w, this.main.y+this.main.h-this.corner.h );
	
	// This line is for resolve a wz_dragdrop bug.
	// When you resize or move en element its defx property isn't
	// updated and you need to update it by hand.
	this.corner.defx = this.main.x+this.main.w-this.corner.w
}

Portlet.prototype.setPopup = function(m) {
	this.popup = m;
	if(m){
		this.corner.maxoffl = 0;
		this.corner.maxofft = 0;
		this.corner.maxoffr = -1;
		this.corner.maxoffb = -1;
	}else{
		this.corner.maxoffl = 0;
		this.corner.maxoffr = 0;
		this.corner.maxofft = 0;
		this.corner.maxoffb = -1;
	}
}

Portlet.prototype.init = function() {
	// Get objects
	this.main = dd.elements["port_"+this.id+"_main"];
	this.title = dd.elements["port_"+this.id+"_title"];
	this.tools = dd.elements["port_"+this.id+"_tools"];
	this.frame = dd.elements["port_"+this.id+"_frame"];
	this.corner = dd.elements["port_"+this.id+"_corner"];

	// Add references
	this.main._portlet = this
	this.title._portlet = this
	this.tools._portlet = this
	this.frame._portlet = this
	this.corner._portlet = this
	
	this.frame.resizeTo( this.main.w, this.main.h-this.title_h );
	this.frame.moveTo( this.main.x, this.main.y+this.title_h+1 );
	this.main.addChild("port_"+this.id+"_frame");

	this.title.resizeTo( this.main.w, this.title_h );
	this.title.moveTo( this.main.x, this.main.y+1 );
	this.main.addChild("port_"+this.id+"_title");

	this.tools.resizeTo( this.tools_w, this.title_h );
	this.tools.moveTo( this.main.x+this.main.w-this.tools_w, this.main.y+1 );
	this.main.addChild("port_"+this.id+"_tools");

	this.corner.moveTo( this.main.x+this.main.w-this.corner.w, this.main.y+this.main.h-this.corner.h );
	this.main.addChild("port_"+this.id+"_corner");

	this.Unblock(this.z);
}

Portlet.prototype.showAll = function() {
	this.main.show();
	this.frame.show();
	this.title.show();
	this.tools.show();
	this.corner.show();
}

Portlet.prototype.setZ = function(val) {
	this.main.setZ(this.main.z+val);
	this.title.setZ(this.main.z-1);
	this.tools.setZ(this.title.z+1);
	this.frame.setZ(this.main.z+1);
	this.corner.setZ(this.main.z+2);
}

Portlet.prototype.Block = function() {
	if(this.popup) v=100
	else v=10
	if(this.dragging) v += 10

	this.frame.setZ(v)
	this.main.setZ(this.frame.z+1);
	this.corner.setZ(this.frame.z+1);
}

Portlet.prototype.Unblock = function() {
	if(this.popup) v=100
	else v=10
	if(this.dragging) v += 10

	this.main.setZ(v);
	this.title.setZ(this.main.z-1);
	this.tools.setZ(this.title.z+1);
	this.frame.setZ(this.main.z+1);
	this.corner.setZ(this.main.z+2);
}

Portlet.prototype.Drag = function(v) {
	if(!this.dragging) {
		this.dragging = true;
		this.Block();
	}
	this.main.resizeTo(dd.obj.x-this.main.x+dd.obj.w, dd.obj.y-this.main.y+dd.obj.h);
	this.title.resizeTo(dd.obj.x-this.main.x+dd.obj.w, this.title_h);
	this.tools.moveTo( this.main.x+this.main.w-this.tools_w, this.main.y+1 );
	this.frame.resizeTo(dd.obj.x-this.main.x+dd.obj.w, dd.obj.y-this.main.y+dd.obj.h-this.title_h);
}

Portlet.prototype.Drop = function(v) {
	this.dragging = false;
	this.Unblock()
}

Portlet.prototype.popUp = function() {
	if(!this.popup){
		this.setPopup(true);
		this.main.resizeTo(dd.getWndW()/2, dd.getWndH()/2);
		this.title.resizeTo(this.main.w, this.title_h);
		this.tools.moveTo( this.main.x+this.main.w-this.tools_w, this.main.y+1 );
		this.frame.resizeTo(this.main.w, this.main.h-this.title_h);
		this.corner.moveTo( this.main.x+this.main.w-this.corner.w, this.main.y+this.main.h-this.corner.h );
		this.main.moveTo((dd.getWndW()-this.main.w)/2,(dd.getWndH()-this.main.h)/2);
		this.Unblock();
	}else{
		this.setPopup(false);
		this.main.resizeTo(dd.getWndW()/2, this.main.defh);
		this.title.resizeTo(this.main.w, this.title_h);
		this.tools.moveTo( this.main.x+this.main.w-this.tools_w, this.main.y+1 );
		this.frame.resizeTo(this.main.w, this.main.h-this.title_h);
	}
}

Portlet.prototype.sendPosition = function() {
	loadXMLDoc('PServlet?action=position&portlet='+this.name+'&row='+this.row+'&col='+this.column.id)
}

Portlet.prototype.sendState = function(state) {
	loadXMLDoc('PServlet?action=state&portlet='+this.name+'&state='+state)
}

Portlet.prototype.sendSize = function() {
	var height = this.getHeight()
	height -= height%1
	loadXMLDoc('PServlet?action=size&portlet='+this.name+'&height='+height)
}

Portlet.prototype.disableDraggable = function() {
	this.Block()
	this.main.setDraggable(false)
}

Portlet.prototype.enableDraggable = function() {
	this.Unblock()
	this.main.setDraggable(true)
}

/*
 * Static objects and functions
 */

var ws = new Workspace();
window.onresize = function() {
	ws.onResize();
};
function my_DragFunc() {
	ws.Drag();
}
function my_DropFunc() {
	ws.Drop();
}
