package org.snap.jounin;

import org.planet.p2pcm.*;

import dermi.*;
import dermi.annotation.DCMImplementation;
import dermi.exception.RemoteException;

import java.util.*;
import java.util.Map.Entry;

@DCMImplementation
public class JouninImpl extends Component implements Jounin, StatefulReplica {
	private static final long serialVersionUID = 872348753;
	
	private Hashtable<String,Participant> participants;
	private Hashtable<String,Portal> portals;
	private Hashtable<String,Long> lastVisitors; 
	
	// Dermi constructor: REQUIRED
	public JouninImpl() {
	}
	
	// Dermi constructor: REQUIRED
	public JouninImpl (Properties env) throws RemoteException {
		super (env);

		participants = new Hashtable<String,Participant>();
		portals = new Hashtable<String,Portal>();
		lastVisitors = new Hashtable<String,Long>();
		
		// Load replica state: as in any other stateful Dermi remote object
		super.loadReplicaState();
	}
	
	public Participant getParticipant(String user) throws RemoteException {
		Participant part = participants.get(user);
		if(part!=null){
			try{
				part.getName();
			}catch(Exception e){
				part = createParticipant(user, "");
			}
		}
		return part;
	}
	
	public static Participant createParticipant(String user, String pass) throws  RemoteException {
		System.out.println("Creating Participant component");
		ParticipantFactory fac;
		try{
			fac = (ParticipantFactory) ComponentUtil.getComponentFactory ("p2p://participant-component.urv.net");
		}catch(Exception e){
			throw new RemoteException("Component factory p2p://participant-component.urv.net not found.");
		}
		System.out.println("I've the factory !");
		Participant part;
		try{
			part = (Participant) fac.createInstance ("org.snap.jounin.Participant", user, user, pass);
		} catch (Exception e) {
			throw new RemoteException("Can not create instance org.snap.jounin.Participant");
		}
		System.out.println("We get the participant");
		return part;
	}
	
	public void addParticipant (Participant part) throws RemoteException {
		if(!participants.containsKey(part.getName())){
			participants.put(part.getName(), part);
		}
	}
	
	public void deleteParticipant(String user) throws RemoteException {
		Participant p = participants.get(user);
		p.delete();
		participants.remove(user);
	}
	
	public boolean existsParticipant(String user) throws RemoteException {
		return participants.containsKey(user);
	}
	
	public Portal getPortal(String name) throws RemoteException {
		Portal port = portals.get(name);
		if(port!=null){
			try{
				port.getName();
			}catch(Exception e){
				port = createPortal(name, "");
			}
		}
		return port;
	}
	
	public static Portal createPortal(String name, String desc) throws RemoteException {
		System.out.println("Creating Portal component");
		PortalFactory fac;
		try {
			fac = (PortalFactory) ComponentUtil.getComponentFactory ("p2p://portal-component.urv.net");
		}catch(Exception e){
			throw new RemoteException("Component factory p2p://portal-component.urv.net not found.");
		}
		System.out.println("I've the factory !");
		Portal port;
		try {
			port = (Portal) fac.createInstance ("org.snap.jounin.Portal", name, name, desc);
		} catch (Exception e) {
			throw new RemoteException("Can not create instance org.snap.jounin.Portal");
		}
		return port;
	}
	
	public void addPortal(Portal port) throws RemoteException {
		System.out.println("We get the participant");
		if(!portals.containsKey(port.getName())){
			portals.put(port.getName(), port);
		}
	}
	
	public void deletePortal(String name) throws RemoteException {
		Portal p = portals.get(name);
		p.delete();
		portals.remove(name);
	}
	
	public boolean existsPortal(String name) throws RemoteException {
		return portals.containsKey(name);
	}
	
	public void loginParticipant(String name) throws RemoteException {
		lastVisitors.put(name, Calendar.getInstance().getTimeInMillis());
	}
	
	private class CompareLastVisits implements Comparator<Map.Entry<String,Long>> {
		public int compare(Map.Entry<String,Long> o1, Map.Entry<String,Long> o2) {
			return o1.getValue().compareTo( o2.getValue() );
		}
	}
	
	public ArrayList getLastVisitors() throws RemoteException {
		Set<Map.Entry<String,Long>> set = lastVisitors.entrySet();
		ArrayList<Map.Entry<String,Long>> keys = new ArrayList<Map.Entry<String,Long>>();
		keys.addAll(set);
		CompareLastVisits comp = new CompareLastVisits();
		Collections.sort(keys, comp);
		Collections.reverse(keys);
		
		//If the lasts list is grather than 10 delete the oldest
		if(keys.size()>10)
		for(int i=(keys.size()-1); i>10; i--) {
			keys.remove(i);
		}
		
		return keys;
	}
	
	/**
	 * The queryInterface method must return a suitable component implementation
	 * for the specified interface
	 * @param interf String The desired interface implementation
	 * @return ComponentInterface The desired implementation instance
	 * @throws InterfaceNotFoundException If the component does not provide an implementation for such specified interface
	 */
	public ComponentInterface queryInterface (String interf) throws InterfaceNotFoundException {
		try {
			if (interf.equals ("org.planet.p2pcm.ComponentInterface") || interf.equals ("org.snap.jounin.Jounin")) {
				return (ComponentInterface) this.copy();
			} else {
				throw new InterfaceNotFoundException ("Interface " + interf + " not implemented.");
			}
		} catch (RemoteException re) {
			throw new InterfaceNotFoundException ("Unable to query for interface " + interf + ": " + re.getMessage());
		}
	}

	/**
	 * Method for loading state into this object
	 */
	public void loadRemoteObjectState() throws RemoteException {
		ArrayList<Object> temp = (ArrayList<Object>) super.loadState();
		participants = (Hashtable) temp.get(0);
		portals = (Hashtable) temp.get(1);
		lastVisitors = (Hashtable) temp.get(2);
	}

	/**
	 * Method for obtaining this object's state
	 */
	public java.io.Serializable getRemoteObjectState() {
		ArrayList<Object> temp = new ArrayList<Object>();
		temp.add (0, participants);
		temp.add (1, portals);
		temp.add (2, lastVisitors);
		return (java.io.Serializable) temp;
	}
}
