package org.snap.jounin;

import org.planet.p2pcm.*;

import dermi.*;
import dermi.exception.*;
import dermi.annotation.*;

import java.util.*;

@DCMImplementation
public class PortalImpl extends Component implements Portal, StatefulReplica {
	private static final long serialVersionUID = 564785653;
	
	private String _name;
	private String _description;
	private Hashtable<String,Rol> _participants;
	private Hashtable<String,String> _resources;
	private Hashtable<String,String> _portlets;
		
	// Dermi constructor: REQUIRED
	public PortalImpl() {
	}
	
	// Dermi constructor: REQUIRED
	public PortalImpl (Properties env) throws RemoteException {
		super (env);
		
		// Load replica state: as in any other stateful Dermi remote object
	    super.loadReplicaState();
	}
	
	// Constructor with arguments
	public PortalImpl (Properties env, String name, String description) throws RemoteException {
		super (env);
		
		_name = name;    
		_description = description;
		_resources = new Hashtable<String,String>();
		_participants = new Hashtable<String,Rol>();
		_portlets = new Hashtable<String,String>();
		
		// Load replica state: as in any other stateful Dermi remote object
		super.loadReplicaState();
		System.out.println ("[Portal] - New portal (" + _name + ") " + " created.");
	}
	
	public String getName() throws RemoteException {
		return _name;
	}
	
	public String getDescription() throws RemoteException {
		return _description;
	}
	
	public void setDescription(String a) throws RemoteException {
		_description = a;
	}
	
	/*
	 * Participants control.
	 */
	public void addParticipant(String name, Rol rol) throws RemoteException {		
		_participants.put(name, rol);
	}
	
	public void delParticipant(String name) throws RemoteException {
		_participants.remove(name);
	}
	
	public Collection getParticipants() throws RemoteException {
		return _participants.keySet();
	}
	
	public Rol getParticipantRol(String part) throws RemoteException {
		if(_participants.containsKey(part))
			return _participants.get(part);
		else
			return Rol.NONE;
	}
	
	public void setParticipantRol(String part, Rol rol) throws RemoteException {
		_participants.remove(part);
		_participants.put(part, rol);
	}
	

	public boolean hasPortlet(String name) throws RemoteException {
		return _portlets.containsKey(name);
	}

	public void addPortlet(String name, String url) throws RemoteException {
		_portlets.put(name, url);
	}

	public void delPortlet(String name) throws RemoteException {
		_portlets.remove(name);
	}
	
	public String getPortletUrl(String name) throws RemoteException {
		return _portlets.get(name);
	}
	
	public Collection getPortlets() throws RemoteException {
		return _portlets.keySet();
	}
	
	public void delete() throws RemoteException {
		try{
			this.passivate();
		}catch(PassivationException e){
			throw new RemoteException("Failed while passivating component portal");
		}
		this.deletePersistentState();
	}

	/**
	 * The queryInterface method must return a suitable component implementation
	 * for the specified interface
	 * @param interf String The desired interface implementation
	 * @return ComponentInterface The desired implementation instance
	 * @throws InterfaceNotFoundException If the component does not provide an implementation for such specified interface
	 */
	public ComponentInterface queryInterface (String interf) throws InterfaceNotFoundException {
		try {
			if (interf.equals ("org.planet.p2pcm.ComponentInterface") || interf.equals ("org.snap.jounin.Portal")) {
				return (ComponentInterface) this.copy();
			} else {
				throw new InterfaceNotFoundException ("Interface " + interf + " not implemented.");
			}
		} catch (RemoteException re) {
			throw new InterfaceNotFoundException ("Unable to query for interface " + interf + ": " + re.getMessage());
		}
	}

	/**
	 * Method for loading state into this object
	 */
	public void loadRemoteObjectState() throws RemoteException {
		ArrayList<Object> temp = (ArrayList<Object>) super.loadState();
		_name = (String) temp.get(0);
		_description = (String) temp.get(1);
		_participants = (Hashtable) temp.get(2);
		_resources = (Hashtable) temp.get(3);
		_portlets = (Hashtable) temp.get(4);
		System.out.println ("[Portal] - state loaded from network.");    
	}

	/**
	 * Method for obtaining this object's state
	 */
	public java.io.Serializable getRemoteObjectState() {
		ArrayList<Object> temp = new ArrayList<Object>();
		temp.add (0, _name);
		temp.add (1, _description);
		temp.add (2, _participants);
		temp.add (3, _resources);
		temp.add (4, _portlets);
		return (java.io.Serializable) temp;
	}
}
